@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sh_instr.c,v 1.51 2007/03/08 19:04:09 debug Exp $
+ *  $Id: cpu_sh_instr.c,v 1.56 2007/04/19 15:49:39 debug Exp $
  *
  *  SH instructions.
  *
@@ -818,7 +818,12 @@ X(mov_l_arg1_postinc_to_arg0_md)
 	else
 		data = BE32_TO_HOST(data);
 	reg(ic->arg[1]) = addr + sizeof(data);
-	reg(ic->arg[0]) = data;
+
+	/*  Special case when loading into the SR register:  */
+	if (ic->arg[0] == (size_t)&cpu->cd.sh.sr)
+		sh_update_sr(cpu, data);
+	else
+		reg(ic->arg[0]) = data;
 }
 X(mov_l_arg1_postinc_to_arg0_fp)
 {
@@ -845,7 +850,8 @@ X(mov_l_arg1_postinc_to_arg0_fp)
 		data = BE32_TO_HOST(data);
 	reg(ic->arg[1]) = addr + sizeof(data);
 
-	if (ic->arg[0] == (size_t)cpu->cd.sh.fpscr)
+	/*  Ugly special case for FPSCR:  */
+	if (ic->arg[0] == (size_t)&cpu->cd.sh.fpscr)
 		sh_update_fpscr(cpu, data);
 	else
 		reg(ic->arg[0]) = data;
@@ -1983,18 +1989,6 @@ X(rts_trace)
 }
 
 
-/*
- *  sts_mach_rn: Store MACH into Rn
- *  sts_macl_rn: Store MACL into Rn
- *  sts_pr_rn:   Store PR into Rn
- *
- *  arg[1] = ptr to rn
- */
-X(sts_mach_rn) { reg(ic->arg[1]) = cpu->cd.sh.mach; }
-X(sts_macl_rn) { reg(ic->arg[1]) = cpu->cd.sh.macl; }
-X(sts_pr_rn)   { reg(ic->arg[1]) = cpu->cd.sh.pr; }
-
-
 /*
  *  rte:  Return from exception.
  */
@@ -2040,7 +2034,7 @@ X(ldtlb)
 			    old_hi & 0xfffff000, INVALIDATE_VADDR);
 		else
 			cpu->invalidate_translation_caches(cpu,
-			    old_hi & 0xfffff000, INVALIDATE_ALL);
+			    0, INVALIDATE_ALL);
 	}
 }
 
@@ -2068,6 +2062,19 @@ X(ldc_rm_sr)
 {
 	RES_INST_IF_NOT_MD;
 	sh_update_sr(cpu, reg(ic->arg[1]));
+
+#if 0
+/*  NOTE: This code causes NetBSD/landisk to get past a point where it
+    otherwise hangs, but it causes Linux/Dreamcast to bug out instead. :/  */
+
+	if (!(cpu->cd.sh.sr & SH_SR_BL) && cpu->cd.sh.int_to_assert > 0 &&
+	    ( (cpu->cd.sh.sr & SH_SR_IMASK) >> SH_SR_IMASK_SHIFT)
+	    < cpu->cd.sh.int_level) {
+		/*  Cause interrupt immediately, by dropping out of the
+		    main dyntrans loop:  */
+		cpu->cd.sh.next_ic = &nothing_call;
+	}
+#endif
 }
 
 
@@ -2191,6 +2198,42 @@ X(ftrc_frm_fpul)
 }
 
 
+/*
+ *  fcnvsd_fpul_drn:  Convert single-precision to double-precision.
+ *  fcnvds_drm_fpul:  Convert double-precision to single-precision.
+ *
+ *  arg[0] = ptr to destination (double- or single-precision float)
+ */
+X(fcnvsd_fpul_drn)
+{
+	struct ieee_float_value op1;
+	int64_t ieee;
+
+	FLOATING_POINT_AVAILABLE_CHECK;
+
+	ieee_interpret_float_value(cpu->cd.sh.fpul, &op1, IEEE_FMT_S);
+	cpu->cd.sh.fpul = (int32_t) op1.f;
+
+	/*  Store double-precision result:  */
+	ieee = ieee_store_float_value(op1.f, IEEE_FMT_D, 0);
+	reg(ic->arg[0]) = (uint32_t) (ieee >> 32);
+	reg(ic->arg[0] + sizeof(uint32_t)) = (uint32_t) ieee;
+}
+X(fcnvds_drm_fpul)
+{
+	struct ieee_float_value op1;
+	int64_t r1;
+
+	FLOATING_POINT_AVAILABLE_CHECK;
+
+	r1 = reg(ic->arg[0] + sizeof(uint32_t)) +
+	    ((uint64_t)reg(ic->arg[0]) << 32);
+	ieee_interpret_float_value(r1, &op1, IEEE_FMT_D);
+
+	cpu->cd.sh.fpul = ieee_store_float_value(op1.f, IEEE_FMT_S, 0);
+}
+
+
 /*
  *  fsca_fpul_drn:  Sinus/cosinus approximation.
  *
@@ -2924,14 +2967,16 @@ X(to_be_translated)
 				}
 				break;
 			case 0x0a:	/*  STS MACH,Rn  */
-				ic->f = instr(sts_mach_rn);
+				ic->f = instr(mov_rm_rn);
+				ic->arg[0] = (size_t)&cpu->cd.sh.mach;
 				break;
 			case 0x12:	/*  STC GBR,Rn  */
 				ic->f = instr(mov_rm_rn);
 				ic->arg[0] = (size_t)&cpu->cd.sh.gbr;
 				break;
 			case 0x1a:	/*  STS MACL,Rn  */
-				ic->f = instr(sts_macl_rn);
+				ic->f = instr(mov_rm_rn);
+				ic->arg[0] = (size_t)&cpu->cd.sh.macl;
 				break;
 			case 0x22:	/*  STC VBR,Rn  */
 				ic->f = instr(copy_privileged_register);
@@ -2948,7 +2993,8 @@ X(to_be_translated)
 				ic->f = instr(movt_rn);
 				break;
 			case 0x2a:	/*  STS PR,Rn  */
-				ic->f = instr(sts_pr_rn);
+				ic->f = instr(mov_rm_rn);
+				ic->arg[0] = (size_t)&cpu->cd.sh.pr;
 				break;
 			case 0x32:	/*  STC SSR,Rn  */
 				ic->f = instr(copy_privileged_register);
@@ -3309,6 +3355,8 @@ X(to_be_translated)
 				ic->arg[1] = (size_t)&cpu->cd.sh.r[r8];	/* n */
 				break;
 			case 0x66:	/*  LDS.L @Rm+,FPSCR  */
+				/*  Note: Loading into FPSCR is a specia
+				    case (need to call sh_update_fpsrc()).  */
 				ic->f = instr(mov_l_arg1_postinc_to_arg0_fp);
 				ic->arg[0] = (size_t)&cpu->cd.sh.fpscr;
 				break;
@@ -3655,6 +3703,14 @@ X(to_be_translated)
 			ic->f = instr(fldi_frn);
 			ic->arg[0] = (size_t)&cpu->cd.sh.fr[r8];
 			ic->arg[1] = 0x3f800000;
+		} else if ((iword & 0x01ff) == 0x00ad) {
+			/*  FCNVSD FPUL,DRn  */
+			ic->f = instr(fcnvsd_fpul_drn);
+			ic->arg[0] = (size_t)&cpu->cd.sh.fr[r8];
+		} else if ((iword & 0x01ff) == 0x00bd) {
+			/*  FCNVDS DRm,FPUL  */
+			ic->f = instr(fcnvds_drm_fpul);
+			ic->arg[0] = (size_t)&cpu->cd.sh.fr[r8];
 		} else if ((iword & 0x01ff) == 0x00fd) {
 			/*  FSCA FPUL,DRn  */
 			ic->f = instr(fsca_fpul_drn);
