@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_cons.c,v 1.37 2006/07/23 14:37:34 debug Exp $
+ *  $Id: dev_cons.c,v 1.40 2007/02/03 20:14:23 debug Exp $
  *  
  *  A simple console device, useful for simple tests.
  *
@@ -42,26 +42,29 @@
 #include "console.h"
 #include "cpu.h"
 #include "device.h"
-#include "devices.h"
 #include "machine.h"
 #include "memory.h"
 #include "misc.h"
 
 #include "testmachine/dev_cons.h"
 
-
 #define	CONS_TICK_SHIFT		14
 
+struct cons_data {
+	int			console_handle;
+	int			in_use;
+	struct interrupt	irq;
+};
+
 
 DEVICE_TICK(cons)
 {
-	struct cpu *c = cpu->machine->cpus[0];
 	struct cons_data *d = extra;
 
-	cpu_interrupt_ack(c, d->irq_nr);
+	INTERRUPT_DEASSERT(d->irq);
 
 	if (console_charavail(d->console_handle))
-		cpu_interrupt(c, d->irq_nr);
+		INTERRUPT_ASSERT(d->irq);
 }
 
 
@@ -135,7 +138,8 @@ DEVINIT(cons)
 	else
 		snprintf(name3, nlen, "%s", devinit->name);
 
-	d->irq_nr = devinit->irq_nr;
+	INTERRUPT_CONNECT(devinit->interrupt_path, d->irq);
+
 	d->in_use = devinit->in_use;
 	d->console_handle = console_start_slave(devinit->machine, name3,
 	    d->in_use);
