@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_le.c,v 1.51 2006/07/14 16:33:28 debug Exp $
+ *  $Id: dev_le.c,v 1.54 2006/12/31 21:35:26 debug Exp $
  *  
  *  LANCE ethernet, as used in DECstations.
  *
@@ -85,7 +85,8 @@ extern int quiet_mode;
 
 
 struct le_data {
-	int		irq_nr;
+	struct interrupt irq;
+	int		irq_asserted;
 
 	uint64_t	buf_start;
 	uint64_t	buf_end;
@@ -537,13 +538,17 @@ static void le_register_fix(struct net *net, struct le_data *d)
 void dev_le_tick(struct cpu *cpu, void *extra)
 {
 	struct le_data *d = (struct le_data *) extra;
+	int new_assert;
 
 	le_register_fix(cpu->machine->emul->net, d);
 
-	if (d->reg[0] & LE_INTR && d->reg[0] & LE_INEA)
-		cpu_interrupt(cpu, d->irq_nr);
-	else
-		cpu_interrupt_ack(cpu, d->irq_nr);
+	new_assert = (d->reg[0] & LE_INTR) && (d->reg[0] & LE_INEA);
+	if (new_assert && !d->irq_asserted)
+		INTERRUPT_ASSERT(d->irq);
+	if (d->irq_asserted && !new_assert)
+		INTERRUPT_DEASSERT(d->irq);
+
+	d->irq_asserted = new_assert;
 }
 
 
@@ -765,7 +770,7 @@ do_return:
  *  dev_le_init():
  */
 void dev_le_init(struct machine *machine, struct memory *mem, uint64_t baseaddr,
-	uint64_t buf_start, uint64_t buf_end, int irq_nr, int len)
+	uint64_t buf_start, uint64_t buf_end, char *irq_path, int len)
 {
 	char *name2;
 	size_t nlen = 55;
@@ -777,7 +782,8 @@ void dev_le_init(struct machine *machine, struct memory *mem, uint64_t baseaddr,
 	}
 
 	memset(d, 0, sizeof(struct le_data));
-	d->irq_nr    = irq_nr;
+
+	INTERRUPT_CONNECT(irq_path, d->irq);
 
 	d->sram = malloc(SRAM_SIZE);
 	if (d->sram == NULL) {
