@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ns16550.c,v 1.55 2006/10/27 13:12:21 debug Exp $
+ *  $Id: dev_ns16550.c,v 1.58 2006/12/30 13:30:58 debug Exp $
  *  
  *  NS16550 serial controller.
  *
@@ -40,6 +40,7 @@
 #include "console.h"
 #include "cpu.h"
 #include "device.h"
+#include "interrupt.h"
 #include "machine.h"
 #include "memory.h"
 #include "misc.h"
@@ -55,11 +56,12 @@
 struct ns_data {
 	int		addrmult;
 	int		in_use;
-	int		irqnr;
 	char		*name;
 	int		console_handle;
 	int		enable_fifo;
 
+	struct interrupt irq;
+
 	unsigned char	reg[DEV_NS16550_LENGTH];
 	unsigned char	fcr;		/*  FIFO control register  */
 	int		int_asserted;
@@ -94,21 +96,18 @@ DEVICE_TICK(ns16550)
 	    ((d->reg[com_ier] & IER_ERXRDY) && (d->reg[com_iir] & IIR_RXRDY))) {
 		d->reg[com_iir] &= ~IIR_NOPEND;
 		if (d->reg[com_mcr] & MCR_IENABLE) {
-			cpu_interrupt(cpu, d->irqnr);
+			INTERRUPT_ASSERT(d->irq);
 			d->int_asserted = 1;
 		}
 	} else {
 		d->reg[com_iir] |= IIR_NOPEND;
 		if (d->int_asserted)
-			cpu_interrupt_ack(cpu, d->irqnr);
+			INTERRUPT_DEASSERT(d->irq);
 		d->int_asserted = 0;
 	}
 }
 
 
-/*
- *  dev_ns16550_access():
- */
 DEVICE_ACCESS(ns16550)
 {
 	uint64_t idata = 0, odata=0;
@@ -332,7 +331,7 @@ DEVINIT(ns16550)
 		exit(1);
 	}
 	memset(d, 0, sizeof(struct ns_data));
-	d->irqnr	= devinit->irq_nr;
+
 	d->addrmult	= devinit->addr_mult;
 	d->in_use	= devinit->in_use;
 	d->enable_fifo	= 1;
@@ -346,6 +345,8 @@ DEVINIT(ns16550)
 	    console_start_slave(devinit->machine, devinit->name2 != NULL?
 	    devinit->name2 : devinit->name, d->in_use);
 
+	INTERRUPT_CONNECT(devinit->interrupt_path, d->irq);
+
 	nlen = strlen(devinit->name) + 10;
 	if (devinit->name2 != NULL)
 		nlen += strlen(devinit->name2);
