@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_uninorth.c,v 1.4 2006/01/01 13:17:18 debug Exp $
+ *  $Id: dev_uninorth.c,v 1.8 2007/02/16 17:17:51 debug Exp $
  *  
  *  Uni-North PCI controller (as used by MacPPC).
  */
@@ -50,9 +50,6 @@ struct uninorth_data {
 };
 
 
-/*
- *  dev_uninorth_addr_access():
- */
 DEVICE_ACCESS(uninorth_addr)
 {
 	struct uninorth_data *d = extra;
@@ -95,9 +92,6 @@ DEVICE_ACCESS(uninorth_addr)
 }
 
 
-/*
- *  dev_uninorth_data_access():
- */
 DEVICE_ACCESS(uninorth_data)
 {
 	struct uninorth_data *d = extra;
@@ -123,7 +117,6 @@ struct pci_data *dev_uninorth_init(struct machine *machine, struct memory *mem,
 	uint64_t addr, int isa_irqbase, int pciirq)
 {
 	struct uninorth_data *d;
-	int pci_irqbase = 0;	/*  TODO  */
 	uint64_t pci_io_offset, pci_mem_offset;
 	uint64_t isa_portbase = 0, isa_membase = 0;
 	uint64_t pci_portbase = 0, pci_membase = 0;
@@ -144,10 +137,10 @@ struct pci_data *dev_uninorth_init(struct machine *machine, struct memory *mem,
 	isa_membase    = 0xd3000000ULL;
 
 	/*  Create a PCI bus:  */
-	d->pci_data = bus_pci_init(machine, pciirq,
+	d->pci_data = bus_pci_init(machine, "ZZZ_irq_stuff",
 	    pci_io_offset, pci_mem_offset,
-	    pci_portbase, pci_membase, pci_irqbase,
-	    isa_portbase, isa_membase, isa_irqbase);
+	    pci_portbase, pci_membase, "XXX_pci_irqbase",
+	    isa_portbase, isa_membase, "YYY_isa_irqbase");
 
 	/*  Add the PCI glue for the controller itself:  */
 	bus_pci_add(machine, d->pci_data, mem, 0, 11, 0, "uninorth");
