@@ -2,7 +2,7 @@
 #define	MISC_H
 
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: misc.h,v 1.247 2006/10/08 02:28:40 debug Exp $
+ *  $Id: misc.h,v 1.251 2007/03/16 14:45:30 debug Exp $
  *
  *  Misc. definitions for gxemul.
  */
@@ -162,6 +162,22 @@ struct memory;
 #define	DEBUG_INDENTATION	4
 
 
+/*  bootblock.c:  */
+int load_bootblock(struct machine *m, struct cpu *cpu,
+	int *n_loadp, char ***load_namesp);
+
+
+/*  bootblock_apple.c:  */
+int apple_load_bootblock(struct machine *m, struct cpu *cpu,
+	int disk_id, int disk_type, int *n_loadp, char ***load_namesp);
+
+
+/*  bootblock_iso9660.c:  */
+int iso_load_bootblock(struct machine *m, struct cpu *cpu,
+	int disk_id, int disk_type, int iso_type, unsigned char *buf,
+	int *n_loadp, char ***load_namesp);
+
+
 /*  dec_prom.c:  */
 int decstation_prom_emul(struct cpu *cpu);
 
@@ -171,6 +187,10 @@ void dreamcast_machine_setup(struct machine *);
 int dreamcast_emul(struct cpu *cpu);
 
 
+/*  dreamcast_scramble.c:  */
+void dreamcast_descramble(char *from, char *to);
+
+
 /*  file.c:  */
 int file_n_executables_loaded(void);
 void file_load(struct machine *machine, struct memory *mem,
@@ -203,6 +223,11 @@ int pc_bios_emul(struct cpu *cpu);
 int playstation2_sifbios_emul(struct cpu *cpu);
 
 
+/*  sh_ipl_g.c:  */
+void sh_ipl_g_emul_init(struct machine *machine);
+int sh_ipl_g_emul(struct cpu *);
+
+
 /*  useremul.c:  */
 void useremul_setup(struct cpu *, int, char **);
 void useremul_syscall(struct cpu *cpu, uint32_t code);
