@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_arc.c,v 1.5 2006/06/30 20:22:54 debug Exp $
+ *  $Id: machine_arc.c,v 1.14 2007/03/24 06:40:15 debug Exp $
  */
 
 #include <stdio.h>
@@ -38,7 +38,6 @@
 #include "device.h"
 #include "devices.h"
 #include "machine.h"
-#include "machine_interrupts.h"
 #include "memory.h"
 #include "misc.h"
 
@@ -48,8 +47,10 @@
 MACHINE_SETUP(arc)
 {
 	struct pci_data *pci_data;
+	void *jazz_data;
 	struct memory *mem = machine->memory;
 	char tmpstr[1000];
+	char tmpstr2[1000];
 	int i, j;
 	char *eaddr_string = "eaddr=10:20:30:40:50:60";		/*  bogus  */
 	unsigned char macaddr[6];
@@ -103,8 +104,12 @@ MACHINE_SETUP(arc)
 		device_add(machine, "sn addr=0x80001000 irq=0");
 		dev_mc146818_init(machine, mem, 0x80004000ULL, 0,
 		    MC146818_ARC_NEC, 1);
-		i = dev_pckbc_init(machine, mem, 0x80005000ULL, PCKBC_8042,
-		    0, 0, machine->use_x11, 0);
+
+fatal("TODO: legacy rewrite\n");
+abort();
+//		i = dev_pckbc_init(machine, mem, 0x80005000ULL, PCKBC_8042,
+//		    0, 0, machine->use_x11, 0);
+i = 0;
 
 		snprintf(tmpstr, sizeof(tmpstr),
 		    "ns16550 irq=3 addr=0x80006000 in_use=%i name2=tty0",
@@ -236,21 +241,29 @@ MACHINE_SETUP(arc)
 			exit(1);
 		}
 
-		machine->md_int.jazz_data = device_add(machine,
-		    "jazz addr=0x80000000");
-		machine->md_interrupt = jazz_interrupt;
+		jazz_data = device_add(machine, "jazz addr=0x80000000");
 
+		/*  Keyboard IRQ is jazz.6, mouse is jazz.7  */
+		snprintf(tmpstr, sizeof(tmpstr),
+		    "%s.cpu[%i].jazz.6", machine->path,
+		    machine->bootstrap_cpu);
+		snprintf(tmpstr2, sizeof(tmpstr2),
+		    "%s.cpu[%i].jazz.7", machine->path,
+		    machine->bootstrap_cpu);
 		i = dev_pckbc_init(machine, mem, 0x80005000ULL,
-		    PCKBC_JAZZ, 8 + 6, 8 + 7,
+		    PCKBC_JAZZ, tmpstr, tmpstr2,
 		    machine->use_x11, 0);
 
+		/*  Serial controllers at JAZZ irq 8 and 9:  */
 		snprintf(tmpstr, sizeof(tmpstr),
-		    "ns16550 irq=16 addr=0x80006000 in_use=%i"
-		    " name2=tty0", machine->use_x11? 0 : 1);
+		    "ns16550 irq=%s.cpu[%i].jazz.8 addr=0x80006000"
+		    " in_use=%i name2=tty0", machine->path,
+		    machine->bootstrap_cpu, machine->use_x11? 0 : 1);
 		j = (size_t)device_add(machine, tmpstr);
 		snprintf(tmpstr, sizeof(tmpstr),
-		    "ns16550 irq=17 addr=0x80007000 in_use=%i"
-		    " name2=tty1", 0);
+		    "ns16550 irq=%s.cpu[%i].jazz.9 addr=0x80007000"
+		    " in_use=0 name2=tty1", machine->path,
+		    machine->bootstrap_cpu);
 		device_add(machine, tmpstr);
 
 		if (machine->use_x11)
@@ -280,118 +293,38 @@ MACHINE_SETUP(arc)
 			break;
 		}
 
-		/*  irq 8 + 4  */
-		device_add(machine, "sn addr=0x80001000 irq=12");
+		/*  SN at JAZZ irq 4  */
+		snprintf(tmpstr, sizeof(tmpstr),
+		    "sn addr=0x80001000 irq=%s.cpu[%i].jazz.4",
+		    machine->path, machine->bootstrap_cpu);
+		device_add(machine, tmpstr);
 
-		dev_asc_init(machine, mem, 0x80002000ULL, 8 + 5, NULL,
-		    DEV_ASC_PICA, dev_jazz_dma_controller,
-		    machine->md_int.jazz_data);
+		/*  ASC at JAZZ irq 5  */
+		snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].jazz.5", 
+		    machine->path, machine->bootstrap_cpu);
+		dev_asc_init(machine, mem, 0x80002000ULL, tmpstr, NULL,
+		    DEV_ASC_PICA, dev_jazz_dma_controller, jazz_data);
 
-		device_add(machine, "fdc addr=0x80003000, irq=0");
+		/*  FDC at JAZZ irq 1  */
+		snprintf(tmpstr, sizeof(tmpstr),
+		    "fdc addr=0x80003000 irq=%s.cpu[%i].jazz.1",
+		    machine->path, machine->bootstrap_cpu);
+		device_add(machine, tmpstr);
 
+		/*  MC146818 at MIPS irq 2:  */
+		snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].2", 
+		    machine->path, machine->bootstrap_cpu);
 		dev_mc146818_init(machine, mem,
-		    0x80004000ULL, 2, MC146818_ARC_JAZZ, 1);
+		    0x80004000ULL, tmpstr, MC146818_ARC_JAZZ, 1);
 
 #if 0
 Not yet.
-		/*  irq = 8+16 + 14  */
+		/*  WDC at ISA irq 14  */
 		device_add(machine, "wdc addr=0x900001f0, irq=38");
 #endif
 
 		break;
 
-	case MACHINE_ARC_JAZZ_M700:
-		/*
-		 *  "Microsoft-Jazz", "Olivetti M700"
-		 *
-		 *  Different enough from Pica and Magnum to be
-		 *  separate here.
-		 *
-		 *  http://mail-index.netbsd.org/port-arc/2000/10/18/0001.html
-		 */
-
-		strlcat(machine->machine_name, " (Microsoft Jazz, "
-		    "Olivetti M700)", MACHINE_NAME_MAXBUF);
-
-		machine->md_int.jazz_data = device_add(machine,
-		    "jazz addr=0x80000000");
-		machine->md_interrupt = jazz_interrupt;
-
-		dev_mc146818_init(machine, mem,
-		    0x80004000ULL, 2, MC146818_ARC_JAZZ, 1);
-
-		i = 0;		/*  TODO: Yuck!  */
-#if 0
-		i = dev_pckbc_init(machine, mem, 0x80005000ULL,
-		    PCKBC_JAZZ, 8 + 6, 8 + 7, machine->use_x11, 0);
-#endif
-
-		snprintf(tmpstr, sizeof(tmpstr), "ns16550 irq=16 addr="
-		    "0x80006000 in_use=%i name2=tty0", machine->use_x11? 0 : 1);
-		j = (size_t)device_add(machine, tmpstr);
-		snprintf(tmpstr, sizeof(tmpstr), "ns16550 irq=17 addr="
-		    "0x80007000 in_use=%i name2=tty1", 0);
-		device_add(machine, tmpstr);
-
-		if (machine->use_x11)
-			machine->main_console_handle = i;
-		else
-			machine->main_console_handle = j;
-
-		dev_m700_fb_init(machine, mem, 0x180080000ULL, 0x100000000ULL);
-
-		break;
-
-	case MACHINE_ARC_DESKTECH_TYNE:
-		/*
-		 *  "Deskstation Tyne" (?)
-		 *
-		 *  TODO
-		 *  http://mail-index.netbsd.org/port-arc/2000/10/14/0000.html
-		 */
-
-		strlcat(machine->machine_name, " (Deskstation Tyne)",
-		    MACHINE_NAME_MAXBUF);
-
-		/*  TODO: IRQs!  */
-		bus_isa_init(machine, 0, 0x900000000ULL,
-		    0x100000000ULL, 8, 24);
-#if 0
-		snprintf(tmpstr, sizeof(tmpstr), "ns16550 irq=0 addr="
-		    "0x9000003f8 in_use=%i name2=tty0", machine->use_x11? 0:1);
-		i = (size_t)device_add(machine, tmpstr);
-		device_add(machine, "ns16550 irq=0 addr=0x9000002f8 in_use=0"
-		    " name2=tty1");
-#endif
-		device_add(machine, "ns16550 irq=0 addr=0x9000003e8 "
-		    "in_use=0 name2=tty2");
-		device_add(machine, "ns16550 irq=0 addr=0x9000002e8 "
-		    "in_use=0 name2=tty3");
-#if 0
-		dev_mc146818_init(machine, mem,
-		    0x900000070ULL, 2, MC146818_PC_CMOS, 1);
-		/*  TODO: irq, etc  */
-		device_add(machine, "wdc addr=0x9000001f0, irq=0");
-		device_add(machine, "wdc addr=0x900000170, irq=0");
-
-		/*  PC kbd  */
-		j = dev_pckbc_init(machine, mem, 0x900000060ULL,
-		    PCKBC_8042, 0, 0, machine->use_x11, 0);
-
-		if (machine->use_x11)
-			machine->main_console_handle = j;
-		else
-			machine->main_console_handle = i;
-#endif
-
-		if (machine->use_x11) {
-			dev_vga_init(machine, mem, 0x1000a0000ULL,
-			    0x9000003c0ULL, machine->machine_name);
-			arcbios_console_init(machine,
-			    0x1000b8000ULL, 0x9000003c0ULL);
-		}
-		break;
-
 	default:fatal("Unimplemented ARC machine type %i\n",
 		    machine->machine_subtype);
 		exit(1);
