@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_hpcmips.c,v 1.3 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_hpcmips.c,v 1.6 2007/01/21 21:02:57 debug Exp $
  */
 
 #include <stdio.h>
@@ -36,7 +36,6 @@
 #include "device.h"
 #include "devices.h"
 #include "machine.h"
-#include "machine_interrupts.h"
 #include "memory.h"
 #include "misc.h"
 
@@ -87,9 +86,7 @@ MACHINE_SETUP(hpcmips)
 		machine->main_console_handle = (size_t)
 		    device_add(machine, tmpstr);
 
-		machine->md_int.vr41xx_data = dev_vr41xx_init(machine,
-		    machine->memory, 4131);
-		machine->md_interrupt = vr41xx_interrupt;
+		dev_vr41xx_init(machine, machine->memory, 4131);
 
 		hpc_platid_cpu_arch = 1;	/*  MIPS  */
 		hpc_platid_cpu_series = 1;	/*  VR  */
@@ -119,9 +116,7 @@ MACHINE_SETUP(hpcmips)
 		machine->main_console_handle = (size_t)
 		    device_add(machine, tmpstr);
 
-		machine->md_int.vr41xx_data = dev_vr41xx_init(machine,
-		    machine->memory, 4121);
-		machine->md_interrupt = vr41xx_interrupt;
+		dev_vr41xx_init(machine, machine->memory, 4121);
 
 		hpc_platid_cpu_arch = 1;	/*  MIPS  */
 		hpc_platid_cpu_series = 1;	/*  VR  */
@@ -145,9 +140,7 @@ MACHINE_SETUP(hpcmips)
 		hpc_fb_bits = 16;
 		hpc_fb_encoding = BIFB_D16_0000;
 
-		machine->md_int.vr41xx_data = dev_vr41xx_init(machine,
-		    machine->memory, 4121);
-		machine->md_interrupt = vr41xx_interrupt;
+		dev_vr41xx_init(machine, machine->memory, 4121);
 
 		hpc_platid_cpu_arch = 1;	/*  MIPS  */
 		hpc_platid_cpu_series = 1;	/*  VR  */
@@ -171,9 +164,7 @@ MACHINE_SETUP(hpcmips)
 		hpc_fb_bits = 16;
 		hpc_fb_encoding = BIFB_D16_0000;
 
-		machine->md_int.vr41xx_data = dev_vr41xx_init(machine,
-		    machine->memory, 4121);
-		machine->md_interrupt = vr41xx_interrupt;
+		dev_vr41xx_init(machine, machine->memory, 4121);
 
 		hpc_platid_cpu_arch = 1;	/*  MIPS  */
 		hpc_platid_cpu_series = 1;	/*  VR  */
@@ -197,9 +188,7 @@ MACHINE_SETUP(hpcmips)
 		hpc_fb_bits = 16;
 		hpc_fb_encoding = BIFB_D16_0000;
 
-		machine->md_int.vr41xx_data = dev_vr41xx_init(machine,
-		    machine->memory, 4121);
-		machine->md_interrupt = vr41xx_interrupt;
+		dev_vr41xx_init(machine, machine->memory, 4121);
 
 		hpc_platid_cpu_arch = 1;	/*  MIPS  */
 		hpc_platid_cpu_series = 1;	/*  VR  */
@@ -223,9 +212,7 @@ MACHINE_SETUP(hpcmips)
 		hpc_fb_bits = 16;
 		hpc_fb_encoding = BIFB_D16_0000;
 
-		machine->md_int.vr41xx_data = dev_vr41xx_init(machine,
-		    machine->memory, 4121);
-		machine->md_interrupt = vr41xx_interrupt;
+		dev_vr41xx_init(machine, machine->memory, 4121);
 
 		hpc_platid_cpu_arch = 1;	/*  MIPS  */
 		hpc_platid_cpu_series = 1;	/*  VR  */
@@ -249,9 +236,7 @@ MACHINE_SETUP(hpcmips)
 		hpc_fb_bits = 4;
 		hpc_fb_encoding = BIFB_D4_M2L_F;
 
-		machine->md_int.vr41xx_data = dev_vr41xx_init(machine,
-		    machine->memory, 4181);
-		machine->md_interrupt = vr41xx_interrupt;
+		dev_vr41xx_init(machine, machine->memory, 4181);
 
 		/*  TODO: Hm... irq 17 according to linux, but
 		    VRIP_INTR_SIU (=9) here?  */
@@ -290,9 +275,7 @@ MACHINE_SETUP(hpcmips)
 		hpc_fb_bits = 16;
 		hpc_fb_encoding = BIFB_D16_0000;
 
-		machine->md_int.vr41xx_data = dev_vr41xx_init(machine,
-		    machine->memory, 4121);
-		machine->md_interrupt = vr41xx_interrupt;
+		dev_vr41xx_init(machine, machine->memory, 4121);
 
 		hpc_platid_cpu_arch = 1;	/*  MIPS  */
 		hpc_platid_cpu_series = 1;	/*  VR  */
@@ -316,11 +299,6 @@ MACHINE_SETUP(hpcmips)
 	    platid_machine, (hpc_platid_vendor << 22) + (hpc_platid_series<<16)
 	    + (hpc_platid_model <<  8) + hpc_platid_submodel);
 
-	if (machine->use_x11) {
-		machine->main_console_handle =
-		    machine->md_int.vr41xx_data->kiu_console_handle;
-	}
-
 	if (hpc_fb_addr != 0) {
 		dev_fb_init(machine, machine->memory, hpc_fb_addr, VFB_HPC,
 		    hpc_fb_xsize, hpc_fb_ysize,
