@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,10 +25,11 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_netwinder.c,v 1.5 2006/08/19 07:58:21 debug Exp $
+ *  $Id: machine_netwinder.c,v 1.13 2007/01/21 21:02:57 debug Exp $
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "bus_isa.h"
@@ -36,13 +37,15 @@
 #include "device.h"
 #include "devices.h"
 #include "machine.h"
-#include "machine_interrupts.h"
 #include "memory.h"
 #include "misc.h"
 
 
 MACHINE_SETUP(netwinder)
 {
+	char tmpstr[300];
+	struct pci_data *pci_bus;
+
 	machine->machine_name = "NetWinder";
 	machine->stable = 1;
 
@@ -53,16 +56,13 @@ MACHINE_SETUP(netwinder)
 	/*  CPU at 63.75 MHz, according to NetBSD's netwinder_machdep.c.  */
 	machine->emulated_hz = 63750000;
 
-	machine->md_int.footbridge_data =
-	    device_add(machine, "footbridge addr=0x42000000");
-	machine->md_interrupt = isa32_interrupt;
-	machine->isa_pic_data.native_irq = 11;
-
-	bus_isa_init(machine, 0, 0x7c000000, 0x80000000, 32, 48);
+	snprintf(tmpstr, sizeof(tmpstr), "footbridge irq=%s.cpu[%i].irq"
+	    " addr=0x42000000", machine->path, machine->bootstrap_cpu);
+	pci_bus = device_add(machine, tmpstr);
 
 	if (machine->use_x11) {
-		bus_pci_add(machine, machine->md_int.footbridge_data->pcibus,
-		    machine->memory, 0xc0, 8, 0, "igsfb");
+		bus_pci_add(machine, pci_bus, machine->memory,
+		    0xc0, 8, 0, "igsfb");
 	}
 
 	if (!machine->prom_emulation)
