@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2006-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_qemu.c,v 1.5 2006/09/04 02:32:00 debug Exp $
+ *  $Id: machine_qemu.c,v 1.12 2007/01/21 21:02:57 debug Exp $
  *
  *  This file contains semi-bogus machine descriptions for experimental
  *  machines, mimicing those emulated by Fabrice Bellard's QEMU.
@@ -35,6 +35,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "bus_isa.h"
@@ -42,7 +43,6 @@
 #include "device.h"
 #include "devices.h"
 #include "machine.h"
-#include "machine_interrupts.h"
 #include "memory.h"
 #include "misc.h"
 
@@ -127,16 +127,19 @@ MACHINE_REGISTER(qemu_arm)
 
 MACHINE_SETUP(qemu_mips)
 {
+	char tmpstr[300];
+
 	machine->machine_name = "QEMU MIPS";
 	cpu->byte_order = EMUL_BIG_ENDIAN;
 
-	/*  An ISA bus, I/O ports at 0x14000000, memory at 0x10000000...  */
-	bus_isa_init(machine, BUS_ISA_IDE0 | BUS_ISA_IDE1,
-	    0x14000000ULL, 0x10000000ULL, 8, 24);
-
-	/*  ... and an ISA interrupt controller, connected to MIPS irq 2:  */
-	machine->md_interrupt = isa8_interrupt;
-	machine->isa_pic_data.native_irq = 2;
+	/*
+	 *  An ISA bus, I/O ports at 0x14000000, memory at 0x10000000,
+	 *  connected to MIPS irq 2:
+	 */
+	snprintf(tmpstr, sizeof(tmpstr), "%s.cpu[%i].2",
+	    machine->path, machine->bootstrap_cpu);
+	bus_isa_init(machine, tmpstr, BUS_ISA_IDE0 | BUS_ISA_IDE1,
+	    0x14000000ULL, 0x10000000ULL);
 
 	if (!machine->prom_emulation)
 		return;
