@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,13 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_sandpoint.c,v 1.3 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_sandpoint.c,v 1.5 2007/02/10 16:29:20 debug Exp $
+ *
+ *  Currently broken; perhaps a problem with NetBSD/sandpoint itself.
+ *  See e.g. http://mail-index.netbsd.org/port-sandpoint/2006/01/02/0000.html
+ *  It seems that .../arch/sandpoint/sandpoint/machdep.c calls
+ *  consinit() before the BATs are initialized, which causes the consinit
+ *  call to fail.
  */
 
 #include <stdio.h>
@@ -41,12 +47,16 @@
 
 MACHINE_SETUP(sandpoint)
 {
+	char tmpstr[300];
+
 	/*
 	 *  NetBSD/sandpoint (http://www.netbsd.org/Ports/sandpoint/)
 	 */
 	machine->machine_name = "Motorola Sandpoint";
 
-	device_add(machine, "ns16550 irq=36 addr=0x800003f8");
+	snprintf(tmpstr, sizeof(tmpstr), "ns16550 irq=%s.cpu[%i] "
+	    "addr=0xfe0003f8", machine->path, machine->bootstrap_cpu);
+	device_add(machine, tmpstr);
 
 	/*  r4 should point to first free byte after the loaded kernel:  */
 	cpu->cd.ppc.gpr[4] = 6 * 1048576;
