@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_shark.c,v 1.3 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_shark.c,v 1.6 2006/12/30 13:31:02 debug Exp $
  */
 
 #include <stdio.h>
@@ -45,7 +45,8 @@ MACHINE_SETUP(shark)
 {
 	machine->machine_name = "Digital DNARD (\"Shark\")";
 
-	bus_isa_init(machine, BUS_ISA_IDE0, 0x08100000, 0xc0000000, 32, 48);
+	bus_isa_init(machine, machine->path,
+	    BUS_ISA_IDE0, 0x08100000, 0xc0000000);
 
 	if (!machine->prom_emulation)
 		return;
